#!/bin/sh

# spip_clone.sh : (C) 2003-2004 Mathieu Lutfy
# Distributed under the terms of the GNU General Public Licence
# $Id: spip_clone.sh,v 1.3 2004/11/03 11:19:00 bgm Exp $

# Creates symlinks to an existing Spip installation
# Usage:
#   $ spip_clone.sh /path/to/existing/spip/  [/path/to/destination/spip/]
# Advanced:
#   $ SPIP_NODB=Yes SPIP_DEBUG=1 spip_clone.sh ...
#
# SPIP_NODB : Does not try to create a database automatically
# SPIP_DEBUG : Prints debug messages
#
# #######################################################################
# CONFIGURATIONS
# #######################################################################

# Dossier par defaut pour les squelettes  (pour mes_fonctions.php3)
DOSSIER_SQUELETTES="squelettes"
VHOSTSBASE="\/var\/www\/vhosts"

APACHEGROUP="apache"

# #######################################################################

help_usage()
{
	echo "Usage:"
	echo "$ new_spip.sh /path/to/existing/spip/  /path/to/destination/spip/"
}

spip_debug()
{
	# To debug, set this while calling, e.g. SPIP_DEBUG=1 spip_clone.sh [...]
	if [ "X$SPIP_DEBUG" != "X" ]; then
		echo $1
	fi
}

symlink_dir()
{
	spip_debug "***********************"
	spip_debug "*** SYMLINK START SRC param = $1"

	local SRCPATH="$1"

	spip_debug "... PWD = `pwd`"

	for i in $SRCPATH/*; do
		SPIPNAME=`echo $i | sed "s@$BASENAME/@@"`

		if [ -f $i ]; then
			# Files
			if [ "$SPIPNAME" == "ecrire/inc_connect.php3" ]; then
				spip_debug "not linking to inc_connect.php3"
			elif [ "$SPIPNAME" == "mes_fonctions.php3" ]; then
				if [ -f mes_fonctions.php3 ]; then
					spip_debug "not touching existing mes_fonctions.php3"
				else
					# we copy to allow the user to customise it
					cp $i .
					mkdir $DOSSIER_SQUELETTES
				fi
			else
				# may exist already (if upgrade)
				ln -s $i 2>/dev/null
			fi 
		elif [ -d $i ]; then
			# Directories
			DIRNAME=`basename $i`

			if [ "$SPIPNAME" == "CACHE" ]; then
				if [ -d CACHE ]; then
					spip_debug "CACHE: le repertoire existe deja (m-a-j)"
				else
					spip_debug "Creation du repertoire 'CACHE'"
					mkdir CACHE
				fi
			elif [ "$SPIPNAME" == "ecrire/AIDE" ]; then
				spip_debug "symlinking to AIDE"
				ln -s $i
			elif [ "$SPIPNAME" == "ecrire/upload" ]; then
				if [ -d upload ]; then
					spip_debug "upload: le repertoire existe deja (m-a-j)"
				else
					spip_debug "Creation du repertoire 'upload'"
					mkdir upload
				fi
			elif [ "$SPIPNAME" == "ecrire/data" ]; then
				if [ -d data ]; then
					spip_debug "data: le repertoire existe deja (m-a-j)"
				else
					spip_debug "Creation du repertoire 'data'"
					mkdir data
				fi
			elif [ "$SPIPNAME" == "ecrire/img_pack" ]; then
				spip_debug "symlinking to ecrire/img_pack"
				ln -s $i
			elif [ "$SPIPNAME" == "IMG/icones_barre" ]; then
				spip_debug "symlinking to IMG/icones_barre"
				ln -s $i
			elif [ "$SPIPNAME" == "trad-lang" ]; then
				spip_debug "trad-lang: directory ignored"
			elif [ "$SPIPNAME" == "spip_clone" ]; then
				spip_debug "spip-clone: directory ignored"
			else
				mkdir $DIRNAME
				cd $DIRNAME

				symlink_dir "$SRCPATH/$DIRNAME"

				cd ..
			fi
		fi
	done

	spip_debug "*** SYMLINK STOP final = `pwd`"
	spip_debug "***********************"

}

# MAIN
spip_debug "** spip_clone main (UID = $UID)"

# Validate 1st parameter (Source path)
if [ "X$1" == "X" ]; then
	help_usage
	exit
else
	SRCPATH="$1"
fi

# Validate 2nd parameter (Destination path)
if [ "X$2" == "X" ]; then
	help_usage
	exit
else
	DESTPATH="$2"

	# We validate the path only if we are running under sudo
	# else it is rather pointless.
	if [ "$UID" == "0" ]; then
		LEGALPATH=`echo $DESTPATH | sed -n "/^$VHOSTSBASE/p"`
	else
		LEGALPATH=$DESTPATH
	fi

	if [ "X$LEGALPATH" == "X" ]; then
		echo "Path is not legal! (Not under $VHOSTSBASE)"
		exit
	fi

	if [ ! -d $DESTPATH ]; then
		mkdir -p $DESTPATH || (echo "Could not create ($DESTPATH)." && exit)
		if [ "X$4" != "X" ]; then
			chown $4 $DESTPATH
			chgrp $4 $DESTPATH
		fi
	fi
fi

# Validate 3rd parameter (Path to templates, may be empty)
if [ "X$3" != "X" ]; then
	if [ -d $3 ]; then
		SQUELPATH="$3"
	else
		echo "Template path is not a directory"
		help_usage
		exit
	fi
else
	unset SQUELPATH
fi

BASENAME="$1"

cd $DESTPATH || exit
symlink_dir $SRCPATH

if [ "X$SQUELPATH" != "X" ]; then
	spip_debug "** Symlinking templates ($SQUELPATH)"
	cd $DESTPATH/squelettes || exit
	symlink_dir $SQUELPATH
else
	spip_debug "** Not symlinking templates, no dir given"
fi


# Lets check for permissions
# #####################################
cd $DESTPATH
spip_debug "** PWD = `pwd`"

# For chmod
if [ "X$SPIP_DEBUG" != "X" ]; then
	VERBOSE="-v"
else
	VERBOSE=""
fi

# Assumes that the apache group is part of the groups having access to the file
spip_debug "Fixing directory and file access modes..."
chmod $VERBOSE 775 CACHE
chmod $VERBOSE 775 ecrire
chmod $VERBOSE 775 ecrire/data
chmod $VERBOSE 775 ecrire/upload
chmod $VERBOSE 775 ecrire/lang
chmod $VERBOSE 775 IMG
chmod $VERBOSE 775 oo
chmod $VERBOSE 775 NAVPICS
chmod $VERBOSE 775 $DOSSIER_SQUELETTES
chmod $VERBOSE 775 mes_fonctions.php3

if [ "$UID" == "0" ]; then
	# i'm running as root, will adapt uid/gid of dirs + permissions

	# Which user owns this directory?
	USER=`stat -t . | cut -d " " -f 5`
	spip_debug "** user is $USER"

	# Fix the rights on important directories:
	spip_debug "Fixing directory and file ownership..."
	chown $USER.$APACHEGROUP CACHE
	chown $USER.$APACHEGROUP ecrire
	chown $USER.$APACHEGROUP ecrire/data
	chown $USER.$APACHEGROUP ecrire/upload
	chown $USER.$APACHEGROUP ecrire/lang/
	chown $USER.$APACHEGROUP IMG
	chown $USER.$USER oo
	chown $USER.$USER NAVPICS
	chown $USER.$USER mes_fonctions.php3
	chown $USER.$USER $DOSSIER_SQUELETTES
fi

# Test if necessary to create the database
# ###############################################################
if [ "$UID" != "0" ]; then
	spip_debug "** script is not running as root, script finished."
	exit;
fi 

if [ "X$SPIP_NODB" != "X" ]; then
	spip_debug "** user has requested not to create DB, script finished"
	exit;
fi

if [ -f ecrire/inc_connect.php3 ]; then
	spip_debug "** We are upgrading SPIP, not touching DB"
	exit;
fi

# Lets create the database
# ###############################################################

# You can force the database name from the command line
if [ "X$MYDBNAME" == "X" ]; then
	# mydbname = /home/path/to/foo => spip_path_to_foo
	MYDBNAME=`pwd | sed s@$VHOSTSBASE@sp@ | sed y@/@_@`
else
	MYDBNAME=`pwd | sed s@$VHOSTSBASE@sp@ | sed s@[^a-zA-Z0-9]@_@g`
fi

DBEXIST=`mysql -e "show databases" | grep $MYDBNAME`

if [ "X$DBEXIST" == "X" ]; then
	# Who is this user ?
	USER=`stat -t . | cut -d " " -f 5`
	USERN=`grep ":$USER:" /etc/passwd | cut -d ":" -f 1`
	spip_debug "** user is $USERN"

	NEWDBPASS="$RANDOM$MYDBNAME$RANDOM"
	echo "** creating database: $MYDBNAME"
	mysql -e "create database $MYDBNAME ;" \
	  && mysql -e "grant all on $MYDBNAME.* to sp_$USERN@localhost identified by '$NEWDBPASS';"

	TMPFILE=`mktemp $0.XXXXXXXX` || exit 1
	echo -e "$MAILINTRO- database = $MYDBNAME,\n- username = sp_$USERN\n- pass = $NEWDBPASS" >> $TMPFILE
	mail -c root -s "New Spip site" $USERN < $TMPFILE
	rm $TMPFILE
fi

## END OF FILE #
